@@ -1,9 +1,9 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
 # INFO:
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
 
 """
 Authors: Evan Hubinger
@@ -11,9 +11,9 @@ License: Apache 2.0
 Description: Handles interfacing with MyPy to make --mypy work.
 """
 
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
 # IMPORTS:
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
 
 from __future__ import print_function, absolute_import, unicode_literals, division
 
@@ -22,6 +22,7 @@ from coconut.root import *  # NOQA
 import traceback
 
 from coconut.exceptions import CoconutException
+from coconut.terminal import logger
 
 try:
     from mypy.api import run
@@ -31,13 +32,14 @@ except ImportError:
         extra="run 'pip install coconut[mypy]' to fix",
     )
 
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
 # CLASSES:
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
 
 
 def mypy_run(args):
     """Runs mypy with given arguments and shows the result."""
+    logger.log_cmd(["mypy"] + args)
     try:
         stdout, stderr, exit_code = run(args)
     except BaseException:
