@@ -1,9 +1,9 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
 # INFO:
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
 
 """
 Authors: Evan Hubinger, Fred Buchanan
@@ -11,9 +11,9 @@ License: Apache 2.0
 Description: Utility functions for the main command module.
 """
 
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
 # IMPORTS:
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
 
 from __future__ import print_function, absolute_import, unicode_literals, division
 
@@ -23,13 +23,13 @@ import sys
 import os
 import traceback
 import subprocess
-import webbrowser
+if PY26:
+    import imp
+from functools import partial
 from copy import copy
 from contextlib import contextmanager
 from select import select
-if PY26:
-    import imp
-else:
+if not PY26:
     import runpy
 try:
     # just importing readline improves built-in input()
@@ -43,11 +43,14 @@ from coconut.constants import (
     main_prompt,
     more_prompt,
     default_style,
-    default_multiline,
-    default_vi_mode,
-    default_mouse_support,
+    default_histfile,
+    prompt_multiline,
+    prompt_vi_mode,
+    prompt_wrap_lines,
+    prompt_history_search,
     style_env_var,
     mypy_path_env_var,
+    histfile_env_var,
     tutorial_url,
     documentation_url,
     reserved_vars,
@@ -55,6 +58,7 @@ from coconut.constants import (
     minimum_recursion_limit,
     oserror_retcode,
     WINDOWS,
+    PY34,
 )
 from coconut.exceptions import (
     CoconutException,
@@ -65,11 +69,26 @@ from coconut.terminal import (
     logger,
     complain,
 )
-from coconut import __coconut__
+
+if PY34:
+    from importlib import reload
+else:
+    from imp import reload
 
 try:
     import prompt_toolkit
+    try:
+        # prompt_toolkit v2
+        from prompt_toolkit.lexers.pygments import PygmentsLexer
+        from prompt_toolkit.styles.pygments import style_from_pygments_cls
+    except ImportError:
+        # prompt_toolkit v1
+        from prompt_toolkit.layout.lexers import PygmentsLexer
+        from prompt_toolkit.styles import style_from_pygments as style_from_pygments_cls
+
     import pygments
+    import pygments.styles
+
     from coconut.highlighter import CoconutLexer
 except ImportError:
     prompt_toolkit = None
@@ -79,9 +98,9 @@ except KeyError:
     ))
     prompt_toolkit = None
 
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
 # FUNCTIONS:
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
 
 
 def openfile(filename, opentype="r+"):
@@ -104,11 +123,13 @@ def readfile(openedfile):
 
 def launch_tutorial():
     """Open the Coconut tutorial."""
+    import webbrowser  # this is expensive, so only do it here
     webbrowser.open(tutorial_url, 2)
 
 
 def launch_documentation():
     """Open the Coconut documentation."""
+    import webbrowser  # this is expensive, so only do it here
     webbrowser.open(documentation_url, 2)
 
 
@@ -215,16 +236,26 @@ def run_file(path):
         return runpy.run_path(path, run_name="__main__")
 
 
-def call_output(cmd, **kwargs):
+def call_output(cmd, stdin=None, encoding_errors="replace", **kwargs):
     """Run command and read output."""
     p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, **kwargs)
     stdout, stderr, retcode = [], [], None
     while retcode is None:
-        out, err = p.communicate()
-        if out is not None:
-            stdout.append(out.decode(get_encoding(sys.stdout)))
-        if err is not None:
-            stderr.append(err.decode(get_encoding(sys.stderr)))
+        if stdin is not None:
+            logger.log_prefix("<0 ", stdin.rstrip())
+        raw_out, raw_err = p.communicate(stdin)
+        stdin = None
+
+        out = raw_out.decode(get_encoding(sys.stdout), encoding_errors) if raw_out else ""
+        if out:
+            logger.log_prefix("1> ", out.rstrip())
+        stdout.append(out)
+
+        err = raw_err.decode(get_encoding(sys.stderr), encoding_errors) if raw_err else ""
+        if err:
+            logger.log_prefix("2> ", err.rstrip())
+        stderr.append(err)
+
         retcode = p.poll()
     return stdout, stderr, retcode
 
@@ -299,26 +330,41 @@ def set_recursion_limit(limit):
     sys.setrecursionlimit(limit)
 
 
-#-----------------------------------------------------------------------------------------------------------------------
-# CLASSES:
-#-----------------------------------------------------------------------------------------------------------------------
+def _raise_ValueError(msg):
+    raise ValueError(msg)
 
 
+def canparse(argparser, args):
+    """Determines if argparser can parse args."""
+    old_error_method = argparser.error
+    argparser.error = _raise_ValueError
+    try:
+        argparser.parse_args(args)
+    except ValueError:
+        return False
+    else:
+        return True
+    finally:
+        argparser.error = old_error_method
+
+
+# -----------------------------------------------------------------------------------------------------------------------
+# CLASSES:
+# -----------------------------------------------------------------------------------------------------------------------
+
 class Prompt(object):
     """Manages prompting for code on the command line."""
     style = None
-    multiline = default_multiline
-    vi_mode = default_vi_mode
-    mouse_support = default_mouse_support
+    multiline = prompt_multiline
+    vi_mode = prompt_vi_mode
+    wrap_lines = prompt_wrap_lines
+    history_search = prompt_history_search
 
     def __init__(self):
         """Set up the prompt."""
         if prompt_toolkit is not None:
-            if style_env_var in os.environ:
-                self.set_style(os.environ[style_env_var])
-            else:
-                self.style = default_style
-            self.history = prompt_toolkit.history.InMemoryHistory()
+            self.set_style(os.environ.get(style_env_var, default_style))
+            self.set_history_file(os.environ.get(histfile_env_var, default_histfile))
 
     def set_style(self, style):
         """Set pygments syntax highlighting style."""
@@ -334,6 +380,13 @@ class Prompt(object):
         else:
             raise CoconutException("unrecognized pygments style", style, extra="use '--style list' to show all valid styles")
 
+    def set_history_file(self, path):
+        """Set path to history file. "" produces no file."""
+        if path:
+            self.history = prompt_toolkit.history.FileHistory(fixpath(path))
+        else:
+            self.history = prompt_toolkit.history.InMemoryHistory()
+
     def input(self, more=False):
         """Prompt for code input."""
         sys.stdout.flush()
@@ -344,7 +397,7 @@ class Prompt(object):
         if self.style is not None:
             internal_assert(prompt_toolkit is not None, "without prompt_toolkit cannot highlight style", self.style)
             try:
-                return prompt_toolkit.prompt(msg, **self.prompt_kwargs())
+                return self.prompt(msg)
             except EOFError:
                 raise  # issubclass(EOFError, Exception), so we have to do this
             except (Exception, AssertionError):
@@ -353,24 +406,35 @@ class Prompt(object):
                 self.style = None
         return input(msg)
 
-    def prompt_kwargs(self):
-        """Get prompt_toolkit.prompt keyword args."""
-        return {
-            "history": self.history,
-            "multiline": self.multiline,
-            "vi_mode": self.vi_mode,
-            "mouse_support": self.mouse_support,
-            "lexer": prompt_toolkit.layout.lexers.PygmentsLexer(CoconutLexer),
-            "style": prompt_toolkit.styles.style_from_pygments(pygments.styles.get_style_by_name(self.style)),
-        }
+    def prompt(self, msg):
+        """Get input using prompt_toolkit."""
+        try:
+            # prompt_toolkit v2
+            prompt = prompt_toolkit.PromptSession(history=self.history).prompt
+        except AttributeError:
+            # prompt_toolkit v1
+            prompt = partial(prompt_toolkit.prompt, history=self.history)
+        return prompt(
+            msg,
+            multiline=self.multiline,
+            vi_mode=self.vi_mode,
+            wrap_lines=self.wrap_lines,
+            enable_history_search=self.history_search,
+            lexer=PygmentsLexer(CoconutLexer),
+            style=style_from_pygments_cls(
+                pygments.styles.get_style_by_name(self.style),
+            ),
+        )
 
 
 class Runner(object):
     """Compiled Python executor."""
 
-    def __init__(self, comp=None, exit=None, store=False, path=None):
+    def __init__(self, comp=None, exit=sys.exit, store=False, path=None):
         """Create the executor."""
-        self.exit = exit if exit is not None else sys.exit
+        # allow direct importing of Coconut files
+        import coconut.convenience  # NOQA
+        self.exit = exit
         self.vars = self.build_vars(path)
         self.stored = [] if store else None
         if comp is not None:
@@ -384,11 +448,13 @@ class Runner(object):
         init_vars = {
             "__name__": "__main__",
             "__package__": None,
+            "reload": reload,
         }
-        for var in reserved_vars:
-            init_vars[var] = None
         if path is not None:
             init_vars["__file__"] = fixpath(path)
+        # put reserved_vars in for auto-completion purposes
+        for var in reserved_vars:
+            init_vars[var] = None
         return init_vars
 
     def store(self, line):
@@ -398,6 +464,7 @@ class Runner(object):
 
     def fix_pickle(self):
         """Fix pickling of Coconut header objects."""
+        from coconut import __coconut__  # this is expensive, so only do it here
         for var in self.vars:
             if not var.startswith("__") and var in dir(__coconut__):
                 self.vars[var] = getattr(__coconut__, var)
