@@ -1,9 +1,9 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
 # INFO:
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
 
 """
 Author: Evan Hubinger
@@ -11,9 +11,9 @@ License: Apache 2.0
 Description: The Coconut IPython kernel.
 """
 
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
 # IMPORTS:
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
 
 from __future__ import print_function, absolute_import, unicode_literals, division
 
@@ -56,11 +56,16 @@ except ImportError:
 else:
     LOAD_MODULE = True
 
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
 # GLOBALS:
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
+
+COMPILER = Compiler(
+    target="sys",
+    line_numbers=True,
+    keep_lines=True,
+)
 
-COMPILER = Compiler(target="sys", line_numbers=True, keep_lines=True)
 RUNNER = Runner(COMPILER)
 
 parse_block_memo = {}
@@ -68,20 +73,19 @@ parse_block_memo = {}
 
 def memoized_parse_block(code):
     """Memoized version of parse_block."""
-    try:
-        result = parse_block_memo[code]
-    except KeyError:
+    success, result = parse_block_memo.get(code, (None, None))
+    if success is None:
         try:
             parsed = COMPILER.parse_block(code)
         except Exception as err:
-            result = err
+            success, result = False, err
         else:
-            result = parsed
-        parse_block_memo[code] = result
-    if isinstance(result, Exception):
-        raise result
-    else:
+            success, result = True, parsed
+        parse_block_memo[code] = (success, result)
+    if success:
         return result
+    else:
+        raise result
 
 
 def memoized_parse_sys(code):
@@ -89,9 +93,9 @@ def memoized_parse_sys(code):
     return COMPILER.header_proc(memoized_parse_block(code), header="sys", initial="none")
 
 
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
 # KERNEL:
-#-----------------------------------------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------------------------------------------------
 
 
 if LOAD_MODULE:
